/*	$LAAS$ */
/*
 * Copyright (c) 2005 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _JLOCOCONST_H
#define _JLOCOCONST_H

#define JLOCO_STRLEN 128

// nominal differential drive parameter values
#define JLOCO_AXIS_WIDTH         0.60
#define JLOCO_RIGHT_WHEEL_RADIUS 0.09
#define JLOCO_LEFT_WHEEL_RADIUS  0.09

#define JLOCO_VMAX_ROBOT 0.8
#define JLOCO_WMAX_ROBOT 1.
#define JLOCO_AMAX_LIN_ROBOT 1.0
#define JLOCO_AMAX_ANG_ROBOT (M_PI / 1.5)

// default correction factors for differential drive model (odometry)
#define JLOCO_AXIS_CORRECTION  1.02 /* 1.01057 */
#define JLOCO_LEFT_CORRECTION  1.01 /* 1.01037 */
#define JLOCO_RIGHT_CORRECTION 1.01 /* 1.01037 */

#define JLOCO_CNTR_POINT_DIST .60
#define JLOCO_CNTR_KP_LONGIT 1.0
#define JLOCO_CNTR_KI_LONGIT 0.0
#define JLOCO_CNTR_KP_TRANSV 1.0
#define JLOCO_CNTR_KI_TRANSV 0.0

// is this even used without a trailer?
#define JLOCO_CNTR_KP_ANG 1.2
#define JLOCO_CNTR_KI_ANG 0.0

#define JLOCO_CNTR_LONGIT_ERROR_MAX 0.1
#define JLOCO_CNTR_TRANSV_ERROR_MAX 0.1
#define JLOCO_CNTR_LONGIT_MAX_ERR_SUM 0.5
#define JLOCO_CNTR_TRANSV_MAX_ERR_SUM 0.5

// is this even used without a trailer?
#define JLOCO_CNTR_ANG_ERROR_MAX 0.15
#define JLOCO_CNTR_ANG_MAX_ERR_SUM 0.3

// longitudinal and transversal tracking errors that trigger the
// "important drift" condition when in position tracking mode (use
// request TrackStart)
#define JLOCO_CNTR_LONGIT_GAP_MAX 1
#define JLOCO_CNTR_TRANSV_GAP_MAX 1

// If jido had a trailer, this value would indicate the angular error
// which triggers an "important drift" condition when in position
// tracking mode. Jido has no trailer... still keep this in here
// because GENPOS_CART_CMD_PARAM is hardwired for robots with
// trailers.
#define JLOCO_CNTR_ANG_GAP_MAX 1.4

#define JLOCO_VAR_XX     0.001
#define JLOCO_VAR_YY     0.001
#define JLOCO_VAR_TT     0.0001
#define JLOCO_VAR_QCOEFF 0.05

#define JLOCO_INFINY_CURV_DTHETA 1e-3
#define JLOCO_INFINY_CURV_RADIUS 10e3

// internal watchdog: number of periods during which it is tolerated
// that a non-zero speed command is not respected (if the robot is
// told to move, but doesn't do so within this number of periods,
// jloco goes into IDLE mode using "internal interrupt")
#define JLOCO_CMD_MISMATCH_THRESHOLD      5

// internal watchdog: number of periods during which it is tolerated
// that the pump task sends non-zero commands while not updating its
// numRef (detects dead pump tasks)
#define JLOCO_PUMP_MISMATCH_THRESHOLD     5

// Small velocity: Values smaller than this are considered 0 when
// transforming speeds from global to actuator space and
// vice-versa. However, for checking if the robot moves when it is
// supposed to, a tenfold higher value is used.
#define JLOCO_EPSILON 0.003

#define JLOCO_JOY_VGAIN (-0.6/32768)
#define JLOCO_JOY_WGAIN (-0.6/32768)

#define JLOCO_MIN_BATTERY_LEVEL 60 /* == 43.5 V */

// execution task periods, one tic is (supposed to be) 10ms
#ifndef FAKE_NEOBOTIX
# define JLOCO_TICS_PERIOD 2
#else
# define JLOCO_TICS_PERIOD 40
#endif

// one period is 10ms, so setting this to 40 is a timeout of 400ms
#define JLOCO_DEFAULT_WATCHDOG_TOLERANCE  40

#define JLOCO_GYRO_MODE_ENABLE  1
#define JLOCO_GYRO_MODE_DISABLE 0
#define JLOCO_DEFAULT_GYRO_MODE JLOCO_GYRO_MODE_DISABLE

#endif
